package org.objectweb.fractal.comete.adl;

import static org.objectweb.fractal.adl.error.ErrorTemplateValidator.validErrorTemplate;

import org.objectweb.fractal.adl.error.ErrorTemplate;

/** {@link ErrorTemplate} group for the idl package. */
public enum DeployErrors implements ErrorTemplate {

  /** */
  PE_MISSING("The processing element must be specified: %s", "file"),

  /** */
  PROTOCOL_MISSING("The protocol must be specified: %s", "file"),

  /** */
  DEFINITION_MISSING("The component must be defined by a definition: %s", "file"),

  /** */
  DIFFERENT_PROTOCOL("The protocols must be the same at %s and %s", "file1", "file2")

  ;

  /** The groupId of ErrorTemplates defined in this enumeration. */
  public static final String GROUP_ID = "Deploy";

  private int                id;
  private String             format;

  private DeployErrors(final String format, final Object... args) {
    this.id = ordinal();
    this.format = format;

    assert validErrorTemplate(this, args);
  }

  public int getErrorId() {
    return id;
  }

  public String getGroupId() {
    return GROUP_ID;
  }

  public String getFormatedMessage(final Object... args) {
    return String.format(format, args);
  }

  public String getFormat() {
    return format;
  }
}
