package org.objectweb.fractal.comete.adl;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.cecilia.adl.CometeLoaderFactory;
import org.objectweb.fractal.comete.gen.DeploymentSources;

/**
 * Abstract class for nodes of the deployment tree
 */
public abstract class ExtendedDeployableNode {

  /**
   * Append sources for this component
   * @param sources the class where the sources must be appent
   * @param parentName the full name of the parent component
   */
  abstract public void genSources(final DeploymentSources sources,
      String parentName);

  /**
   * Compile all the required binaries
   * @param factory the factory that must be used to compile ADL files
   * @param context the compiler context (contains its configuration)
   * @param soDir the directory where binaries must be put
   * @param buildDirectory the directory that must be used for temporary files
   * @param definitionsDone the set of definitions already compiled
   * @return the new set of comiled definitions 
   * @throws ADLException
   */
  abstract public Set<String> compile(final CometeLoaderFactory factory,
      final Map<Object, Object> context, final String soDir, final String buildDirectory,
      Set<String> definitionsDone) throws ADLException;
  

  /**
   * Compile all the required binaries
   * @param factory the factory that must be used to compile ADL files
   * @param context the compiler context (contains its configuration)
   * @param soDir the directory where binaries must be put
   * @param buildDirectory the directory that must be used for temporary files
   * @return the new set of comiled definitions 
   * @throws ADLException
   */
  public Set<String> compile(final CometeLoaderFactory factory,
      final Map<Object, Object> context, final String soDir, final String buildDirectory) throws ADLException {
    Set<String> definitionsDone = new HashSet<String>();
    return compile(factory, context, soDir, buildDirectory, definitionsDone);
  }
  
}
