
package org.objectweb.fractal.comete.adl.bindings;

import java.io.File;
import java.util.Map;
import java.util.Set;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.cecilia.adl.CometeLoaderFactory;
import org.objectweb.fractal.comete.adl.ExtendedDeployableNode;
import org.objectweb.fractal.comete.gen.DeploymentSources;

/**
 * This class describes a deployable binding
 */
public class ExtendedDeployableBinding extends ExtendedDeployableNode {

  /** The AST node of the binding */
  DeployableBinding binding;

  /** The constructor
   * @param binding the AST node of the binding
   */
  public ExtendedDeployableBinding(final DeployableBinding binding) {
    this.binding = binding;
  }

  /** 
   * Get the AST node of the binding
   * @return the AST node
   */
  public DeployableBinding getBinding() {
    return binding;
  }

  public void genSources(final DeploymentSources sources,
      String parentName) {
    // Nothing to generate
  }

  /**
   * Compile the stubs and skeletons required for this binding
   */
  @Override
  public Set<String> compile(final CometeLoaderFactory factory,
      final Map<Object, Object> context, final String soDir, final String buildDirectory,
      final Set<String> definitionsDone) throws ADLException {

    /*
     * Compile stub/skeleton for each binding with a protocol
     */
    final String signature = (String) binding.astGetDecoration("signature");

    // Check if we already compile for this signature and this protocol
    final String protocol = binding.getProtocol();
    String fullDef = signature + "." + protocol;
    if (!definitionsDone.contains(fullDef)) {

      definitionsDone.add(fullDef);

      // Determine the paths
      final String stubsoname = protocol.toUpperCase() + "_STUB_"
          + signature.replace(".", "_");
      final String skelsoname = protocol.toUpperCase() + "_SKEL_"
          + signature.replace(".", "_");
      final String stubsopath = soDir + stubsoname;
      final String skelsopath = soDir + skelsoname;

      // Determine the definition names
      final String stubDef = "minus.bare.generic.exportbind." + protocol
          + ".lib.Stub(" + signature + ")";

      final String skelfDef = "minus.bare.generic.exportbind." + protocol
          + ".lib.Skeleton(" + signature + ")";

      // Compile
      factory.compile(stubDef, context, stubsopath, buildDirectory
          + File.separator + stubsoname);
      factory.compile(skelfDef, context, skelsopath, buildDirectory
          + File.separator + skelsoname);
    }
    return definitionsDone;
  }
}
