
package org.objectweb.fractal.comete.adl.components;

/**
 * An AST node interface to define a component with deployment
 * specification (processing element, start order, ...)
 */
public interface DeployableComponent
    extends
      org.objectweb.fractal.adl.components.Component {
  /**
   * Get the processor element where the component must be deployed
   * 
   * @return the processor element
   */
  String getPe();

  /**
   * Set the processor element where the component must be deployed
   * 
   * @param arg the processor element
   */
  void setPe(String arg);

  /**
   * Set the start order of the component (used to known in which order the
   * sub-component of a composite must be started)
   * 
   * @return the start order
   */
  String getStartOrder();

  /**
   * Get the start order of the component (used to known in which order the
   * sub-component of a composite must be started)
   * 
   * @param arg the start order
   */
  void setStartOrder(String arg);

  /**
   * Get the template of the component (system, loader, ...)
   * @return the template
   */
  String getTemplate();

  /**
   * Set the template of the component
   * @param arg the template
   */
  void setTemplate(String arg);

}
