
package org.objectweb.fractal.comete.adl.components;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Map;
import java.util.Set;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.util.FractalADLLogManager;
import org.objectweb.fractal.cecilia.adl.CometeLoaderFactory;
import org.objectweb.fractal.comete.adl.ExtendedDeployableNode;
import org.objectweb.fractal.comete.gen.DeploymentSources;

/**
 * Abstract class for component nodes of the deployment tree
 */
public abstract class ExtendedDeployableComponent
    extends
      ExtendedDeployableNode {

  protected String             structName = null;
  String loader = null;

  // TODO should be of type DeploymentComponent
  protected ComponentContainer componentContainer;

  /**
   * Constructor
   * 
   * @param componentContainer the component container of the component
   */
  public ExtendedDeployableComponent(ComponentContainer componentContainer) {
    this.componentContainer = componentContainer;
  }

  /**
   * Generate all required source files
   * 
   * @param context the compiler context (contains compiler configuration)
   */
  public void genSources(Map<Object, Object> context) {
    final DeploymentSources sources = new DeploymentSources();
    this.genSources(sources, null);

    if (sources.hasDeployer()) {
      // Generate the deployer source file
      final File outputDir = (File) context.get("adlBuildDirectory");
      try {
        final FileWriter output = new FileWriter(outputDir.getAbsoluteFile()
            + File.separator + "deployerStructs.c");
        output.write(sources.getDeployerSource());
        output.close();
      } catch (final IOException e) {
        FractalADLLogManager.STEP.info(e.toString());
      }

      // Append the file in the class path
      URLClassLoader cl = (URLClassLoader) context.get("classloader");
      final URL newUrls[] = new URL[cl.getURLs().length + 1];
      System.arraycopy(cl.getURLs(), 0, newUrls, 1, cl.getURLs().length);
      try {
        newUrls[0] = outputDir.toURI().toURL();
      } catch (final MalformedURLException e) {
        // never append
        throw new Error(e);
      }
      cl = URLClassLoader.newInstance(newUrls);
      context.put("classloader", cl);
    }

    if (!sources.hasLoader()) {
        this.loader = sources.generateCometeLoader((Definition)componentContainer, context);
    }
  }

  @Override
  public Set<String> compile(final CometeLoaderFactory factory,
      final Map<Object, Object> context, final String soDir, final String buildDirectory,
      Set<String> definitionsDone) throws ADLException {
    if (loader != null) {
      factory.compile(loader, context, soDir + "loader", buildDirectory);
    }
    return null;
  }
}
