
package org.objectweb.fractal.comete.adl.components;

import java.util.Map;
import java.util.Set;

import org.objectweb.fractal.comete.adl.DeployErrors;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.cecilia.adl.CometeLoaderFactory;
import org.objectweb.fractal.comete.gen.DeploymentPrimitiveStruct;
import org.objectweb.fractal.comete.gen.DeploymentSources;
import org.objectweb.fractal.comete.improveast.DeployAstUtil;

/**
 * Abstract class for primitive nodes of the deployment tree
 */
public class ExtendedDeployablePrimitive extends ExtendedDeployableComponent {

  DeployableComponent dc;

  ExtendedDeployablePrimitive(final DeployableComponent dc) throws ADLException {
    super(dc);
    this.dc = dc;
  }

  @Override
  public void genSources(final DeploymentSources sources,
      final String parentName) {
    // Don't generate any structure for the loader
    if (!DeployAstUtil.isLoaderComponent(dc)) {
      final String defName = ((String) dc.astGetDecoration("definition"))
          .replace(".", "_");
      final String name = dc.getName();
      String fullName;
      // Get the full name
      if (parentName == null)
        fullName = "";
      else if ("".equals(parentName))
        fullName = name;
      else
        fullName = parentName + "_" + name;

      structName = fullName;

      final DeploymentPrimitiveStruct struct = new DeploymentPrimitiveStruct(
          fullName, defName, structName, dc.getPe());
      sources.addComponentSource(fullName, struct.getSource());
    } else
      sources.addLoader();
  }

  @Override
  public Set<String> compile(final CometeLoaderFactory factory,
      final Map<Object, Object> context, final String soDir,
      final String buildDirectory, final Set<String> definitionsDone)
      throws ADLException {

    super.compile(factory, context, soDir, buildDirectory, definitionsDone);

    final String definition = (String) dc.astGetDecoration("definition");
    if (definition == null) {
      throw new ADLException(DeployErrors.DEFINITION_MISSING, dc
          .astGetSource());
    
    }
    
    String execName = null;

    // Respect the convention for the loader name
    if (DeployAstUtil.isLoaderComponent(dc))
      execName = "loader";
    else
      execName = definition.replace('.', '_');

    // Only compile if it has not been compiled yet
    if (!definitionsDone.contains(definition)) {
      definitionsDone.add(definition);
      factory.compile(definition, context, soDir + execName, buildDirectory);
    }

    return definitionsDone;
  }
}
