
package org.objectweb.fractal.comete.improveast;

import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.bindings.Binding;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.interfaces.Interface;
import org.objectweb.fractal.cecilia.improveast.AstUtil;
import org.objectweb.fractal.comete.adl.bindings.DeployableBinding;
import org.objectweb.fractal.comete.adl.components.DeployableComponent;

/**
 * A utilitary class to ease Comete AST manipulation
 */
public class DeployAstUtil
    extends
      org.objectweb.fractal.cecilia.improveast.AstUtil {

  /**
   * Tell if the given node is a deployable component (component that can be
   * deployed through Comete
   * 
   * @param arg the node
   * @return true if it is a deployable component
   */
  public static boolean isDeployableComponent(final Node arg) {
    return (arg instanceof DeployableComponent && ((DeployableComponent) arg)
        .getPe() != null);
  }

  /**
   * Get the server interface bounded to the given client interface
   * 
   * @param arg the client interface
   * @return the server interface
   */
  public static Interface getDeployableServerInterface(Node arg) {

    do {
      if (isClientInterface(arg)) {
        /* itf is client */
        arg = getServerItf(((Binding) arg.astGetDecoration("bind")));
      } else if (isServerInterface(arg)) {
        /* itf is server */
        if (isDeployableComponent(getParent(arg))) {
          return (Interface) arg;
        } else if (isPrimitive(getParent(arg))) {
          return null;
        } else {
          arg = getServerItf((Binding) arg.astGetDecoration("bind"));
        }
      }
    } while (arg != null);
    return null;
  }

  /**
   * Get the binding of the given interface
   * 
   * @param itf the interface
   * @return the binding
   */
  public static DeployableBinding getDeployableBinding(Interface itf) {

    do {
      final Binding binding = (Binding) itf.astGetDecoration("bind");
      if (binding instanceof DeployableBinding) {
        return (DeployableBinding) binding;
      }
      itf = (Interface) binding.astGetDecoration("server");

    } while (itf != null);
    return null;
  }

  /**
   * Tell if the given binding is a system one (i.e. either the client or the
   * server is the system component
   * 
   * @param binding the binding
   * @return true if the binding is a system one
   */
  public static boolean isSystemBinding(final Binding binding) {
    boolean isSystem = true;
    // If the client is connected to the 'system' component
    // this means it is a system binding.
    // The binding can be between collection interfaces
    // In this case all connected components must be system components
    for (ComponentContainer serverCc:AstUtil.getServerComponent(binding)) {
      if (!isSystemComponent(serverCc))
        isSystem = false;
    }
    return isSystem;
  }

  /**
   * Tell if the given component is the system component
   * @param arg the component
   * @return true if the component is the system component
   */
  public static boolean isSystemComponent(final Node arg) {
    if (arg instanceof DeployableComponent) {
      final DeployableComponent dc = (DeployableComponent) arg;
      return "system".equals(dc.getTemplate());
    }
    return false;
  }

  /**
   * Tell if the given component is the loader component
   * @param arg the component
   * @return true if the component is the loader component
   */
  public static boolean isLoaderComponent(final Node arg) {
    if (arg instanceof DeployableComponent) {
      final DeployableComponent dc = (DeployableComponent) arg;
      return "loader".equals(dc.getTemplate());
    }
    return false;
  }

}
