
package org.objectweb.fractal.comete.improveast;

import java.util.List;

import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.interfaces.Interface;
import org.objectweb.fractal.cecilia.improveast.GenericImprover;
import org.objectweb.fractal.cecilia.improveast.Improver;
import org.objectweb.fractal.comete.adl.bindings.DeployableBinding;
import org.objectweb.fractal.comete.adl.components.DeployableComponent;

/**
 * Improve given AST.
 * <p>
 * The AST is enriched with the following decorations:
 * <ul>
 * <li>signature (for Binding nodes): the signature of the interfaces connected
 * to the binding
 * <li>deployable-server-interface (for client Interface nodes): the server
 * interface connected to this client interface
 * </ul>
 */
public class DeploymentImprover extends GenericImprover {

  /**
   * Deployment Improver constructor. Registers AST improvers.
   */
  public DeploymentImprover() {
    super();

    registerImprover(DeployableBinding.class, new Improver() {
      public List<Node> improve(final Node arg) {
        return DeploymentImprover
            .improveDeployableBinding((DeployableBinding) arg);
      }
    });

    registerImprover(DeployableComponent.class, new Improver() {
      public List<Node> improve(final Node arg) {
        return DeploymentImprover.improveComponent((DeployableComponent) arg);
      }
    });

    registerImprover(Interface.class, new Improver() {
      public List<Node> improve(final Node arg) {
        return DeploymentImprover.improveInterface((Interface) arg);
      }
    });
  }

  private static List<Node> improveDeployableBinding(final DeployableBinding arg) {
    Node parent;
    Node from;
    Interface ifrom;

    // Add the signature

    parent = DeployAstUtil.getParent(arg);

    from = DeployAstUtil.getComponent(parent, arg.getFrom().split("\\.")[0]);
    ifrom = DeployAstUtil.getInterface(from, arg.getFrom().split("\\.")[1]);

    arg.astSetDecoration("signature", DeployAstUtil.getSignature(ifrom));

    return null;
  }

  private static List<Node> improveComponent(final DeployableComponent arg) {

    if (arg.getStartOrder() == null) {
      arg.setStartOrder("0");
    }

    if (arg.getTemplate() != null) {
      // Some templates has an implicit PE that we must fill
      if ("loader".equals(arg.getTemplate())) {
        arg.setPe("0");
      }
    }

    return null;
  }

  /**
   * Improve definition
   */
  private static List<Node> improveInterface(final Interface arg) {
    return null;
  }

}
